# Generated by Django 4.0.2 on 2022-07-14 19:53

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('shaarpy', '0005_alter_links_date_created'),
    ]

    operations = [
        migrations.AlterField(
            model_name='links',
            name='date_created',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



# coding: utf-8
"""
    ShaarPy
"""
from django.test import TestCase
from shaarpy.models import Links


class LinksTest(TestCase):

    """
        Links Model
    """

    def create_link(self):

        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True

        return Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky)

    def test_folders(self):
        inst = self.create_link()
        self.assertTrue(isinstance(inst, Links))



# coding: utf-8
"""
    ShaarPy :: AppConfig
"""

from django.apps import AppConfig


class ShaarpyConfig(AppConfig):
    name = 'shaarpy'
    verbose_name = "ShaarPy"
    default_auto_field = 'django.db.models.AutoField'



# coding: utf-8
"""
    ShaarPy :: Importing in Netscape HTML File
"""

from django.core.management.base import BaseCommand
from rich.console import Console
from shaarpy.tools import import_shaarli

console = Console()

__author__ = 'FoxMaSk'


def load(html_file, reload_article_from_url):
    """
        load an HTML Nestcape file
    """
    if html_file.endswith('.html'):
        import_shaarli(html_file, reload_article_from_url)


class Command(BaseCommand):
    help = 'Import HTML Shaarli export file'

    def add_arguments(self, parser):
        parser.add_argument("file", help="provide the path to the HTML file to import", type=str)
        parser.add_argument("--reload", help="if you want to reload the article from the website", action='store_true')

    def handle(self, *args, **options):
        console.print(f"Shaarpy :: Importing file {options['file']} in progress", style="green")
        load(options['file'], options['reload'])
        console.print(f"Shaarpy :: Importing file {options['file']} is finished", style="green")



# coding: utf-8
"""
    ShaarPy :: Forms
"""
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _
from django.forms import ModelForm, TextInput, Textarea, CheckboxInput, EmailInput, HiddenInput
from shaarpy.models import Links
from shaarpy.tools import url_cleaning


class LinksForm(ModelForm):
    """
        form to create / edit a note / link
    """

    class Meta:

        model = Links
        fields = ('url', 'title', 'image', 'text', 'tags', 'private', 'sticky')
        widgets = {
            'tags': TextInput(attrs={'class': 'form-control', 'placeholder': _('tags,tag2')}),
            'url': TextInput(attrs={'class': 'form-control',
                                    'placeholder': _('Drop an URL or leave if empty for creating a note')}),
            'title': TextInput(attrs={'class': 'form-control', 'placeholder': _('Note:')}),
            'image': TextInput(attrs={'class': 'form-control', 'placeholder': _('Image URL:')}),
            'text': Textarea(attrs={'class': 'form-control', 'placeholder': _('content')}),
            'private': CheckboxInput(attrs={'class': 'form-check-input'}),
            'sticky': CheckboxInput(attrs={'class': 'form-check-input'}),
            'url_hashed': HiddenInput(),
        }

    def clean(self):
        cleaned_data = super().clean()
        url = cleaned_data.get("url")
        title = cleaned_data.get("title")
        text = cleaned_data.get("text")

        if url is None and title is None and text == '':
            msg = _("You need to enter something in one of those fields 'URL' or 'Title' or 'Text'")
            self.add_error('url', msg)
            self.add_error('title', msg)
            self.add_error('text', msg)

    def clean_url(self):
        """
            remove unwanted query parameters
        """
        data = self.cleaned_data['url']
        return url_cleaning(data)

    def clean_tags(self):
        """
            remove extra space
        """
        unwanted_chars = '?./:;!#&@{}[]|$`\\^~*+=-_'
        data = self.cleaned_data['tags']
        if data:
            if any(s in unwanted_chars for s in data):
                msg = _(f"characters {unwanted_chars} not allowed.")
                self.add_error('tags', msg)

            if data.endswith(','):
                data = data[:-1]
            return data.replace(' ', '')


class MeForm(ModelForm):

    """
        form to edit the user's profile
    """

    class Meta:

        model = User
        fields = ('email', 'last_name')
        widgets = {
            'last_name': TextInput(attrs={'class': 'form-control', 'placeholder': _('Last name')}),
            'email': EmailInput(attrs={'class': 'form-control', 'placeholder': _('Email')}),
        }



# coding: utf-8
"""
    ShaarPy
"""
import os
from django.conf import settings
from django.test import TestCase


class SettingsTestCase(TestCase):

    """
      check that all the needed config is present
    """

    def test_env_file(self):
        """
        check existing .env file
        """
        assert os.path.isfile('shaarpy/.env'), True

    def test_get_config_service(self):
        """
        check expecting config
        """
        self.assertIs(type(settings.SHAARPY_NAME), str)
        self.assertIs(type(settings.SHAARPY_DESCRIPTION), str)
        self.assertIs(type(settings.SHAARPY_AUTHOR), str)
        self.assertIs(type(settings.SHAARPY_ROBOT), str)
        self.assertIs(type(settings.SHAARPY_LOCALSTORAGE_MD), str)
        self.assertIs(type(settings.SHAARPY_STYLE), str)
        self.assertIs(type(settings.LANGUAGE_CODE), str)
        self.assertIs(type(settings.TIME_ZONE), str)
        self.assertIs(type(settings.USE_TZ), bool)
        self.assertIs(type(settings.ALLOWED_HOSTS), list)
        self.assertIs(type(settings.SECRET_KEY), str)
        self.assertIs(type(settings.CSRF_TRUSTED_ORIGINS), list)



# coding: utf-8
"""
    ShaarPy
"""
from datetime import date, timedelta, datetime, timezone

from django.test import RequestFactory, TestCase
from django.contrib.auth.models import AnonymousUser, User
from django.urls import reverse

from shaarpy.forms import LinksForm
from shaarpy.models import Links
from shaarpy.views import LinksList, LinksCreate, LinksDelete, LinksDetail, LinksUpdate
from shaarpy.views import DailyLinks, TagsList, LinksByTagList, PrivateLinks, PublicLinks


class CommonStuffTestCase(TestCase):
    """
    Common fixtures
    """

    def create_link(self):
        """
        create a link
        """
        url = 'https://foxmask.eu.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home,sweet,'

        return Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags)

    def setUp(self):
        super(CommonStuffTestCase, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username='foxmask', email='my@email.org', password='top_secret')


class LinksDeleteTestCase(CommonStuffTestCase):
    """
    Deal with deletion
    """

    def test_delete(self):
        link = self.create_link()
        template = "shaarpy/links_confirm_delete.html"
        # Setup request and view.
        request = RequestFactory().get(f'delete/{link.id}')
        request.user = self.user

        view = LinksDelete.as_view(template_name=template)
        # Run.
        response = view(request, user=request.user, pk=link.id)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class LinksListTestCase(CommonStuffTestCase):
    """
    Home View for the current user
    """

    def test_all_links_list(self):
        self.create_link()
        template = "shaarpy/links_list.html"
        # Setup request and view.
        request = RequestFactory().get('/')
        request.user = self.user

        view = LinksList.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)

    def test_search(self):
        self.create_link()
        template = "shaarpy/links_list.html"
        # Setup request and view.
        request = RequestFactory().get('/?q=foobar')
        request.user = self.user

        view = LinksList.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)

    def test_close_bookmarklet(self):
        self.create_link()
        template = "shaarpy/links_list.html"
        # Setup request and view.
        request = RequestFactory().get('/?source=bookmarklet')
        request.user = self.user

        view = LinksList.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class LinksListAnonymousTestCase(CommonStuffTestCase):
    """
    Home View for anonymous user
    """

    def setUp(self):
        super(LinksListAnonymousTestCase, self).setUp()
        self.factory = RequestFactory()

    def test_all_links_list(self):
        self.create_link()
        template = "shaarpy/links_list.html"
        # Setup request and view.
        request = RequestFactory().get('/')
        request.user = AnonymousUser()

        view = LinksList.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class PrivateLinksTestCase(CommonStuffTestCase):
    """
    Private Links for the current user
    """

    def test_private_links_list(self):
        self.create_link()
        template = "shaarpy/links_list.html"
        # Setup request and view.
        request = RequestFactory().get('/links/private/')
        request.user = self.user

        view = PrivateLinks.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class PublicLinksTestCase(CommonStuffTestCase):
    """
    Public Links for the current user
    """

    def test_public_links_list(self):
        self.create_link()
        template = "shaarpy/links_list.html"
        # Setup request and view.
        request = RequestFactory().get('/links/public/')
        request.user = self.user

        view = PublicLinks.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class CreateLinksTestCase(CommonStuffTestCase):
    """
    Create Links from the form
    """
    def setUp(self):
        super(CommonStuffTestCase, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username='foxmask', email='my@email.org', password='top_secret')

    def create_link(self):
        """
        create a link
        """
        url = 'https://foxmask.eu.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home,sweet,'

        return Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags)

    def test_get_the_form(self):
        template = "link_form.html"
        # Setup request and view.
        url_to_create = 'https://foxmask.eu.org/feeds/all.rss.xml'
        title = 'Le Free de la passion'
        url_data = f"?post={url_to_create}&title={title}&source=bookmarklet"
        request = RequestFactory().get(reverse('link_create') + url_data)
        request.user = self.user
        view = LinksCreate.as_view(template_name=template)
        # Run.

        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)

    def test_create_link(self):
        url_to_create = 'https://foxmask.eu.org/a-propos.html#a-propos'
        data = {
            'url': url_to_create,
            'title': '',
            'text': '',
            'tags': 'home,sweet,',
        }
        request = RequestFactory().post(reverse('link_create'), data=data)
        request.user = self.user
        view = LinksCreate.as_view()
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 302)

    def test_create_link_duplicate(self):
        # check to avoid creating duplicate of the same URL
        self.create_link()
        data = {
            'url': 'https://foxmask.eu.org/',
            'title': 'Le Free de la passion',
            'text': '# Le Free de la Passion',
            'private': False,
            'sticky': True,
            'tags': 'home,sweet,'
        }
        request = RequestFactory().post(reverse('link_create'), data=data)
        request.user = self.user
        view = LinksCreate.as_view()
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 302)

    def test_create_note(self):
        # Setup request and view.
        url_to_create = 'https://foxmask.eu.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la passion'
        data = {
            'url': url_to_create,
            'title': title,
            'text': text,
            'tags': 'home,sweet,',
        }
        request = RequestFactory().post(reverse('link_create'), data=data)
        request.user = self.user
        view = LinksCreate.as_view()
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 302)

    def test_create_note_wo_title(self):
        # Setup request and view.
        text = '# Le Free de la passion'
        data = {
            'title': '',
            'text': text,
        }
        request = RequestFactory().post(reverse('link_create'), data=data)
        request.user = self.user
        view = LinksCreate.as_view()
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 302)

    def create_links(self):
        url = 'https://foxmask.eu.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home,sweet,'
        return Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags)

    def test_create3_drop_image(self):
        # create an entry into the model
        self.create_links()

        url = 'http://world.kbs.co.kr/service/index.htm?lang=e'
        title = 'KBS World'
        text = '# KBS World'
        private = False
        sticky = True
        tags = 'Korea'
        data = {
            'url': url,
            'title': title,
            'text': text,
            'private': private,
            'sticky': sticky,
            'tags': tags,
        }
        # try to create an entry from the form but with same URL
        request = RequestFactory().post(reverse('link_create'), data=data)
        request.user = self.user
        view = LinksCreate.as_view()
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 302)

    def test_form_valid(self):
        data = {'url': 'https://foxmask.eu.org/', 'tags': 'home,sweet,'}
        form = LinksForm(data=data)
        self.assertTrue(form.is_valid())

    def test_form_valid_url_cleaning(self):
        data = {'url': 'https://foxmask.eu.org/?utm_source=foo&utm_source=bar', 'tags': 'home,sweet,'}
        form = LinksForm(data=data)
        self.assertTrue(form.is_valid())

    def test_form_valid2(self):
        data = {'title': 'My note', 'text': 'note text'}
        form = LinksForm(data=data)
        self.assertTrue(form.is_valid())

    def test_form_invalid(self):
        data = {'url': '', 'title': '', 'text': ''}
        form = LinksForm(data=data)
        self.assertFalse(form.is_valid())

    def test_form_invalid2(self):
        data = {'title': 'My note', 'text': 'note text', 'tags': '@toto'}
        form = LinksForm(data=data)
        self.assertFalse(form.is_valid())


class LinksDetailTestCase(CommonStuffTestCase):
    """
    Deal with Links detail page
    """

    def test_link_detail(self):
        link = self.create_link()
        template = "shaarpy/links_detail.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('link_detail', kwargs={'slug': link.url_hashed}))
        view = LinksDetail.as_view(template_name=template)
        # Run.
        response = view(request, pk=link.id)   # @TODO why pk=link.id and not slug=link.url_hashed ?
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class LinksUpdateTestCase(CommonStuffTestCase):
    """
    Deal with Link updating
    """

    def test_link_update(self):
        link = self.create_link()
        template = "shaarpy/links_form.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('link_edit', kwargs={'pk': link.id}))
        request.user = self.user
        view = LinksUpdate.as_view(template_name=template)
        # Run.
        response = view(request, pk=link.id)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class LinksByTagListTestCase(CommonStuffTestCase):
    """
    Deal with displaying Links for a given tag
    """

    def test_one_tag(self):
        self.create_link()
        template = "links_list.html"
        # Setup request and view.
        tags = 'foobar'

        request = RequestFactory().get(reverse('links_by_tag_list', kwargs={'tags': tags}))
        request.user = self.user

        view = LinksByTagList.as_view(template_name=template)
        # Run.
        response = view(request, tags=tags)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)

    def test_no_tag(self):
        self.create_link()

        template = "links_list.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('links_by_tag_list', kwargs={'tags': '0Tag'}))
        tags = '0Tag'
        request.user = self.user
        view = LinksByTagList.as_view(template_name=template)
        # Run.
        response = view(request, tags=tags)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)

    def test_public_links(self):
        # user object AnonymousUser + private = False
        self.create_link()
        template = "links_list.html"
        # Setup request and view.
        tags = 'home'

        request = RequestFactory().get(reverse('links_by_tag_list', kwargs={'tags': tags}))
        request.user = AnonymousUser()

        view = LinksByTagList.as_view(template_name=template)
        # Run.
        response = view(request, tags=tags)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class TagsListTestCase(TestCase):
    """
    display the list of tags
    """

    def setUp(self):
        super(TagsListTestCase, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username='foxmask', email='my@email.org', password='top_secret')

    def create_links(self):
        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home'
        return Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags)

    def create_links_no_tags(self):
        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        return Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky)

    def test_tag_index(self):
        self.create_links()
        self.create_links_no_tags()
        template = "tags_list.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('tags_list'))
        request.user = self.user
        view = TagsList.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class DailyListAnonymousTestCase(CommonStuffTestCase):
    """
    Home View for anonymous user
    """

    def setUp(self):
        super(DailyListAnonymousTestCase, self).setUp()
        self.factory = RequestFactory()

    def create_links_yesterday(self):
        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home'
        today = datetime.now(timezone.utc).astimezone()
        yesterday = today - timedelta(days=1)
        Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags,
                             date_created=yesterday)

    def create_links_today(self):
        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home'
        today = datetime.now(timezone.utc).astimezone()
        Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags,
                             date_created=today)

    def test_daily(self):
        self.create_links_today()
        template = "daily_list.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('daily'))
        view = DailyLinks.as_view(template_name=template)
        request.user = AnonymousUser()
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)

    def test_previous(self):
        self.create_links_yesterday()
        today = date.today()
        yesterday = today - timedelta(days=1)
        template = "daily_list.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('daily'), kwargs={'yesterday': str(yesterday)})
        view = DailyLinks.as_view(template_name=template)
        request.user = AnonymousUser()
        # Run.
        response = view(request, yesterday=str(yesterday))
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class DailyLinksTestCase(TestCase):
    """
    Deal with Daily links
    """

    def setUp(self):
        super(DailyLinksTestCase, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username='foxmask', email='my@email.org', password='top_secret')

    def create_links_yesterday(self):
        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home'
        today = datetime.now(timezone.utc).astimezone()
        yesterday = today - timedelta(days=1)
        Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags,
                             date_created=yesterday)

    def create_links_today(self):
        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home'
        today = datetime.now(timezone.utc).astimezone()
        Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags,
                             date_created=today)

    def test_daily(self):
        self.create_links_today()
        template = "daily_list.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('daily'))
        view = DailyLinks.as_view(template_name=template)
        request.user = self.user
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)

    def test_previous(self):
        self.create_links_yesterday()
        today = date.today()
        yesterday = today - timedelta(days=1)
        template = "daily_list.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('daily'), kwargs={'yesterday': str(yesterday)})
        view = DailyLinks.as_view(template_name=template)
        request.user = self.user
        # Run.
        response = view(request, yesterday=str(yesterday))
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)



"""
URL configuration for shaarpy project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.contrib.auth import views as auth_views
from django.contrib.sitemaps.views import sitemap
from django.urls import path, re_path
from shaarpy import settings
from shaarpy.sitemaps import ShaarpySitemap
from shaarpy.views import Me, MeUpdate
from shaarpy.views import (LinksList, LinksCreate, LinksUpdate, LinksDetail, LinksDelete)
from shaarpy.views import (LinksByTagList, PublicLinks, PrivateLinks, TagsList, DailyLinks)
from shaarpy.views import LatestLinksFeed

urlpatterns = [
    path('admin/', admin.site.urls),

    path('accounts/login/', auth_views.LoginView.as_view(
        extra_context={'SHAARPY_NAME': settings.SHAARPY_NAME}), name="login"),
    path('accounts/profile/', Me.as_view(), name="me"),
    path('accounts/profile/edit/', MeUpdate.as_view(), name='edit_me'),

    path('', LinksList.as_view(), name="home"),
    path('new/', LinksCreate.as_view(), name='link_create'),
    path('edit/<int:pk>/', LinksUpdate.as_view(), name='link_edit'),
    path('link/<slug:slug>/', LinksDetail.as_view(), name='link_detail'),
    re_path(r'delete/(?P<pk>\d+)$', LinksDelete.as_view(), name='link_delete'),
    path('tags/', TagsList.as_view(), name='tags_list'),
    re_path(r'tags/(?P<tags>\w+)$', LinksByTagList.as_view(), name='links_by_tag_list'),
    path('daily/', DailyLinks.as_view(), name='daily'),
    path('links/private/', PrivateLinks.as_view(), name='link_private'),
    path('links/pubic/', PublicLinks.as_view(), name='link_public'),
    re_path(r'daily/(?P<yesterday>\d\d\d\d-\d\d-\d\d)', LinksList.as_view(), name='daily'),

    path('feed/', LatestLinksFeed(), name='feed'),

    path('sitemap.xml', sitemap,
         {'sitemaps': {'shaarpy': ShaarpySitemap}},
         name='django.contrib.sitemaps.views.sitemap'),
]

handler403 = 'shaarpy.views.error_403'
handler404 = 'shaarpy.views.error_404'
handler500 = 'shaarpy.views.error_500'



# Generated by Django 4.0.1 on 2022-01-23 10:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shaarpy', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='links',
            options={'ordering': ['-sticky', '-date_created'], 'verbose_name_plural': 'Links'},
        ),
        migrations.AddField(
            model_name='links',
            name='sticky',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 4.0.1 on 2022-01-26 12:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shaarpy', '0002_alter_links_options_links_sticky'),
    ]

    operations = [
        migrations.AddField(
            model_name='links',
            name='url_hashed',
            field=models.URLField(blank=True, max_length=2048, null=True),
        ),
        migrations.AlterField(
            model_name='links',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# coding: utf-8
"""
Django settings for ShaarPy project.

Generated by 'django-admin startproject' using Django 4.0.1.

For more information on this file, see
https://docs.djangoproject.com/en/4.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.0/ref/settings/
"""

from pathlib import Path
import environ
import os

env = environ.Env()

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

env_file_path = os.path.join(BASE_DIR, 'shaarpy', '.env')
env_file = environ.Env.read_env(env_file_path)
env.read_env(env_file)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = env.str('SECRET_KEY', default='!DONTFORGETTOCHANGETHISVALUE!')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = env.bool('DEBUG', default=False)  # set to False when using in production

ALLOWED_HOSTS = env.list('ALLOWED_HOSTS', default=['127.0.0.1', 'localhost'])

# Application definition

INSTALLED_APPS = [
    'shaarpy.apps.ShaarpyConfig',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

if DEBUG:
    INSTALLED_APPS += [
        "debug_toolbar",
    ]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
]

if DEBUG:
    MIDDLEWARE = [
        'debug_toolbar.middleware.DebugToolbarMiddleware',
    ]

MIDDLEWARE += [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.cache.FetchFromCacheMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'shaarpy.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'shaarpy.wsgi.application'


# Database
# https://docs.djangoproject.com/en/4.0/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': env.str('DB_ENGINE', default='django.db.backends.sqlite3'),
        'NAME': env.str('DB_NAME', default=BASE_DIR / 'db.sqlite3'),
        'USER': env.str('DB_USER', default=''),
        'PASSWORD': env.str('DB_PASSWORD', default=''),
        'HOST': env.str('DB_HOST', default=''),
        'PORT': env.str('DB_PORT', default=''),
    },
    'TEST': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'nyuseu-test.sqlite3',
    }
}


# Password validation
# https://docs.djangoproject.com/en/4.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

SHAARPY_CACHE_DIR = env.str('SHAARPY_CACHE_DIR', default='/tmp')
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': SHAARPY_CACHE_DIR,
    }
}

CACHE_MIDDLEWARE_ALIAS = 'default'
CACHE_MIDDLEWARE_SECONDS = 600

# Internationalization
# https://docs.djangoproject.com/en/4.0/topics/i18n/
LANGUAGE_CODE = env.str('LANGUAGE_CODE', default='en-en')
TIME_ZONE = env.str('TIME_ZONE', default='UTC')
USE_I18N = env.bool('USE_I18N', default=True)
USE_TZ = env.bool('USE_TZ', default=True)


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.0/howto/static-files/

STATIC_URL = 'static/'
STATIC_ROOT = BASE_DIR / 'static'

# Default primary key field type
# https://docs.djangoproject.com/en/4.0/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

CSRF_TRUSTED_ORIGINS = env.list('CSRF_TRUSTED_ORIGINS')

LOGOUT_REDIRECT_URL = '/'

SHAARPY_AUTHOR = env.str('SHAARPY_AUTHOR', default="FoxMaSk")
SHAARPY_NAME = env.str('SHAARPY_NAME', default=f"ShaarPy - {SHAARPY_AUTHOR} Links")
SHAARPY_DESCRIPTION = env.str('SHAARPY_DESCRIPTION', default="Share thoughts, links ideas, notes")
SHAARPY_ROBOT = env.str('SHAARPY_ROBOT', default="index, follow")
SHAARPY_LOCALSTORAGE_MD = env.str('SHAARPY_LOCALSTORAGE_MD')
SHAARPY_STYLE = env.str('SHAARPY_STYLE', default="blue")

# debugtoolbar

INTERNAL_IPS = [
    # ...
    "127.0.0.1",
    # ...
]



# coding: utf-8
"""
    ShaarPy Test Feed
"""

from django.test import RequestFactory, TestCase
from shaarpy.models import Links
from shaarpy.views import LatestLinksFeed


class LatestLinksFeedTestCase(TestCase):
    """
    test Feed
    """

    def create_links(self):
        """
        fixture
        """
        url = 'https://foxmask.org/'
        title = 'Le Free de la passion'
        text = '# Le Free de la Passion'
        private = False
        sticky = True
        tags = 'home,sweet,'
        return Links.objects.create(url=url, title=title, text=text, private=private, sticky=sticky, tags=tags)

    def test_linksfeed(self):
        """
        lets test the link feed
        """
        # Setup request and view.
        self.create_links()
        request = RequestFactory().get('feed')
        view = LatestLinksFeed()
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)



# coding: utf-8
"""
    ShaarPy :: Models
"""
from django.core.validators import RegexValidator
from django.db import models
from django.utils import timezone
import pytz

# Regex to match latin and 한글 (hangul) chars
alphanum_hangul = RegexValidator(
    r'^[0-9a-zA-Z\u3131-\uD79D\u1100-\u11FF\u3130-\u318F\uA960-\uA97F\uAC00-\uD7AF\uD7B0-\uD7FF,]*$',
    'Only alphanumeric characters are allowed.')


class Links(models.Model):
    """
    Links model to handle URLs / notes
    """
    tags = models.CharField(max_length=255, null=True, blank=True, validators=[alphanum_hangul])
    url = models.URLField(max_length=2048, null=True, blank=True)
    url_hashed = models.SlugField(max_length=10, null=True, blank=True)
    title = models.CharField(max_length=255, null=True, blank=True)
    text = models.TextField(null=True, blank=True)
    private = models.BooleanField(default=False)
    image = models.TextField(null=True, blank=True)
    video = models.TextField(null=True, blank=True)
    date_created = models.DateTimeField(default=timezone.now)
    sticky = models.BooleanField(default=False)

    class Meta:
        """
        Meta properties
        """
        verbose_name_plural = "Links"
        ordering = ['-sticky', '-date_created']



# coding: utf-8
"""
    ShaarPy :: Importing Pelican Markdown file
"""

from django.core.management.base import BaseCommand
import os
from rich.console import Console
from shaarpy.tools import import_pelican

console = Console()

__author__ = 'FoxMaSk'


def load(md_file):
    """
        load Pelican Markdown file
    """
    if md_file.endswith('.md'):
        import_pelican(md_file)


class Command(BaseCommand):
    help = 'Import Pelican Markdown file'

    def add_arguments(self, parser):
        parser.add_argument("file", help="provide the path to the Markdown file to import", type=str)

    def handle(self, *args, **options):
        console.print(f"Shaarpy :: Importing file {options['file']} in progress", style="green")
        load(options['file'])
        console.print(f"Shaarpy :: Importing file {options['file']} is finished", style="green")



# coding: utf-8
"""
    ShaarPy :: Tools

    - Importing/Exporting in Netscape HTML File
    - Load article from url with image/video
    - Manage Markdown file creation
"""
import base64
import copy
from datetime import datetime
import html
import logging
import os
from pathlib import Path
import re
from typing import NoReturn
from urllib.parse import urlparse
from slugify import slugify

from bs4 import BeautifulSoup
from django.utils import timezone
from django.utils.text import Truncator
from jinja2 import Environment, PackageLoader
from newspaper import Article

import newspaper
import pypandoc
from rich.console import Console
from rich.table import Table
from shaarpy.models import Links
from shaarpy import settings

console = Console()
logger = logging.getLogger('tools')
"""
    URL
"""


def url_cleaning(url: str) -> str:
    """
    drop unexpected content of the URL from the bookmarklet

    param url: url of the website
    :return string url
    """

    if url:
        for pattern in ('&utm_source=', '?utm_source=', '&utm_medium=', '#xtor=RSS-'):
            pos = url.find(pattern)
            if pos > 0:
                url = url[0:pos]
    return url


# ARTICLES MANAGEMENT


def _get_host(url: str) -> str:
    """
    go to get the schema and hostname and port related to the given url

    param url: url of the website
    :return string 'hostname'
    """

    o = urlparse(url)
    hostname = o.scheme + '://' + o.hostname
    port = ''
    if o.port is not None and o.port != 80:
        port = ':' + str(o.port)
    hostname += port
    return hostname


def _get_brand(url: str) -> str:
    """
    go to get the brand name related to the given url

    url: url of the website
    :return string of the Brand
    """
    brand = newspaper.build(url=_get_host(url))
    brand.download()
    brand.parse()
    return brand.brand


def drop_image_node(content: str) -> tuple:
    """
    drop the image node if found

    content: content of the html possibly containing the img
    return the first found image and the content
    """
    my_image = ''
    soup = BeautifulSoup(content, 'html.parser')
    if soup.find_all('img', src=True):
        image = soup.find_all('img', src=True)[0]
        my_image = copy.copy(image['src'])
        # if not using copy.copy(image) before
        # image.decompose(), it drops content of the 2 vars
        # image and my_image
        image.decompose()
    return my_image, soup


def grab_full_article(url: str) -> tuple:
    """
        get the complete article page from the URL
    url: URL of the article to get
    return title text image video or the URL in case of ArticleException
    """
    # get the complete article
    r = Article(url, keep_article_html=True)
    try:
        r.download()
        r.parse()
        article_html = r.article_html
        video = r.movies[0] if len(r.movies) > 0 else ''
        image = ''
        # check if there is a top_image
        if r.top_image:
            # go to check image in the article_html and grab the first one found in article_html
            # it may happened that top_image is not the same in the content of article_html
            # so go pickup this one and remove it in the the content of article_html
            image, article_html = drop_image_node(article_html)
        # convert into markdown
        output = Truncator(article_html).chars("400", html=True)
        text = pypandoc.convert_text(output, 'md', format='html')
        title = r.title + ' - ' + _get_brand(url)

        return title, text, image, video
    except newspaper.article.ArticleException:
        return url, "", "", ""


# MARKDOWN MANAGEMENT


def rm_md_file(title: str) -> NoReturn:
    """
        rm a markdown file
    title: the name of the file to remove
    """
    file_name = slugify(title) + '.md'
    file_md = f'{settings.SHAARPY_LOCALSTORAGE_MD}/{file_name}'
    if os.path.exists(file_md):
        os.remove(file_md)


def create_md_file(storage: str, title: str, url: str, text: str,
                   tags: str, date_created: str, private: bool, image: str, video: str) -> NoReturn:
    """
        create a markdown file
    storage: path of the folder where to store the file
    title: title of the file
    url: url of the article
    text: text of the article
    tags: tags if provided
    date_created: creation date
    private: boolean true/false
    image: the main image if any
    video: the main video if any
    """

    data = {'title': title,
            'url': url,
            'text': text,
            'date': date_created,
            'private': private,
            'tags': tags,
            'image': image,
            'video': video,
            'author': settings.SHAARPY_AUTHOR,
            'style': settings.SHAARPY_STYLE}

    env = Environment(
        loader=PackageLoader('shaarpy', 'templates'), autoescape=True
    )
    template = env.get_template('shaarpy/shaarpy_markdown.md')
    output = template.render(data=data)
    file_name = slugify(title) + '.md'
    file_md = f'{storage}/{file_name}'
    # overwrite existing file with same slug name
    with open(file_md, 'w') as ls:
        ls.write(output)


# CRC Stuff


def crc_that(string: str) -> int:
    """
    the PHP's hash(crc32) in Python :P

    implem in python:
       https://chezsoi.org/shaarli/shaare/U7admg
       https://stackoverflow.com/a/50843127/636849
    """
    a = bytearray(string, "utf-8")
    crc = 0xffffffff
    for x in a:
        crc ^= x << 24
        for k in range(8):
            crc = (crc << 1) ^ 0x04c11db7 if crc & 0x80000000 else crc << 1
    crc = ~crc
    crc &= 0xffffffff
    return int.from_bytes(crc.to_bytes(4, 'big'), 'little')


def small_hash(text: str) -> str:
    """
    Returns the small hash of a string, using RFC 4648 base64url format
   eg. smallHash('20111006_131924') --> yZH23w
   Small hashes:
     - are unique (well, as unique as crc32, at last)
     - are always 6 characters long.
     - only use the following characters: a-z A-Z 0-9 - _ @
     - are NOT cryptographically secure (they CAN be forged)
    In Shaarli, they are used as a tinyurl-like link to individual entries.
    """
    number = crc_that(text)

    number_bytes = number.to_bytes((number.bit_length() + 7) // 8, byteorder='big')

    encoded = base64.b64encode(number_bytes)
    final_value = encoded.decode().rstrip('=').replace('+', '-').replace('/', '_')
    return final_value


# IMPORTING SHAARLI FILE

def import_shaarli(the_file: str, reload_article_from_url: str) -> NoReturn:  # noqa: C901
    """
    the_file: name of the file to import
    reload_article_from_url: article url
    """
    private = 0
    with open(the_file, 'r') as f:
        data = f.read()
        msg = f"ShaarPy :: importing {the_file}"
        logger.debug(msg)

    if data.startswith('<!DOCTYPE NETSCAPE-Bookmark-file-1>'):
        i = 0
        table = Table(show_header=True, header_style="bold magenta")

        table.add_column("Title", style="cyan")
        table.add_column("Private", style="yellow")
        table.add_column("Date", style="dim")

        for html_article in data.split('<DT>'):
            i += 1
            link = {'url': '',
                    'title': '',
                    'text': '',
                    'tags': '',
                    'image': None,
                    'video': None,
                    'date_created': '',
                    'private': False}
            if i == 1:
                continue

            if len(html_article.split('<DD>')) == 2:
                line, text = html_article.split('<DD>')
                link['text'] = html.unescape(text)

            for line in html_article.split('<DD>'):
                if line.startswith('<A '):
                    matches = re.match(r"<A (.*?)>(.*?)</A>", line)

                    attrs = matches[1]

                    link['title'] = matches[2] if matches[2] else ''
                    link['title'] = html.unescape(link['title'])

                    for attr in attrs.split(" "):
                        matches = re.match(r'([A-Z_]+)="(.+)"', attr)
                        attr_found = matches[1]
                        value_found = matches[2]
                        if attr_found == 'HREF':
                            link['url'] = html.unescape(value_found)
                        elif attr_found == 'ADD_DATE':
                            raw_add_date = int(value_found)
                            if raw_add_date > 30000000000:
                                raw_add_date /= 1000
                            link['date_created'] = datetime.fromtimestamp(raw_add_date).replace(tzinfo=timezone.utc)
                        elif attr == 'PRIVATE':
                            link['private'] = 0 if value_found == '0' else 1
                        elif attr == 'TAGS':
                            link['tags'] = value_found

                    if link['url'] != '' and link['url']:

                        if reload_article_from_url:
                            if link['url'].startswith('?'):
                                continue
                            link['title'], link['text'], link['image'], link['video'] = grab_full_article(link['url'])

                        if private:
                            link['private'] = 1

                        table.add_row(link['title'],
                                      "Yes" if link['private'] else "No",
                                      str(link['date_created']))

                        try:
                            obj = Links.objects.get(url=link['url'])
                            obj.title = link['title']
                            obj.text = link['text']
                            obj.tags = link['tags']
                            obj.private = private
                            obj.date_created = link['date_created']
                            obj.image = link['image']
                            obj.video = link['video']
                            obj.url_hashed = small_hash(link['date_created'].strftime("%Y%m%d_%H%M%S"))
                            msg = f"ShaarPy :: updating {obj.url}"
                            logger.debug(msg)
                            obj.save()
                        except Links.DoesNotExist:
                            new_values = {'url': link['url'],
                                          'url_hashed': small_hash(link['date_created'].strftime("%Y%m%d_%H%M%S")),
                                          'title': link['title'],
                                          'text': link['text'],
                                          'tags': link['tags'],
                                          'private': private,
                                          'date_created': link['date_created'],
                                          'image': link['image'],
                                          'video': link['video'],
                                          }
                            obj = Links(**new_values)
                            obj.save()
                            msg = f"ShaarPy :: creating {obj.url}"
                            logger.debug(msg)

        console.print(table)

# IMPORTING PELICAN FILE


def import_pelican(the_file: str) -> NoReturn:  # noqa: C901
    """
    Headers are :

    Title: Home Sweet Home
    Date: 2021-09-27
    Author: foxmask
    Category: Korea
    Tags: hosting, Korea
    Slug: home-sweet-home
    Status: published
    Summary: text

    body content

    the_file: path of the file to create
    """
    private = 0
    title = ''
    date_created = ''
    slug = ''
    tags = ''
    url_hashed = ''
    text = ''
    status = ''
    summary = ''
    author = ''

    with open(the_file, 'r') as f:
        data = f.readlines()
        msg = f"ShaarPy :: importing {the_file}"
        logger.debug(msg)

        for line in data:
            if line.startswith('Author:'):
                author = line.split('Author: ')[1].strip()
                author = f"</br>By {author}"
            if line.startswith("Status: "):
                status = line.split('Status: ')[1].strip()
            if line.startswith("Title: "):
                title = line.split('Title: ')[1].strip()
            if line.startswith("Date: "):
                date_created = line.split('Date: ')[1].strip()
                if len(date_created) == 10:
                    # date without hours minutes secondes
                    date_created += ' 00:00:00'
                elif len(date_created) == 16:
                    # date with hours minutes
                    date_created += ':00'
                date_created = datetime.strptime(date_created, "%Y-%m-%d %H:%M:%S")

            if line.startswith("Tags: "):
                tags = line.split('Tags: ')[1].strip()

                unwanted_chars = '?./:;!#&@{}[]|`\\^~*+=-_'

                if any(s in unwanted_chars for s in tags):
                    tags = ''

                if tags.endswith(','):
                    tags = tags[:-1]
                tags = tags.replace(' ', '')

            if line.startswith("Slug: "):
                slug = line.split('Slug: ')[1].strip()
            if line.startswith("Summary: "):
                summary = '# ' + line.split('Summary: ')[1] + "\n\n"
            if status == 'published':
                url_hashed = small_hash(date_created.strftime("%Y%m%d_%H%M%S"))
            if not line.startswith(("Status:", "Title:", "Date:", "Tags:", "Slug", 'Status:', 'Summary:')):
                text += summary + line
    if status == 'published':

        try:
            Links.objects.get(url_hashed=url_hashed)
            console.print(f"Shaarpy :: {title} already exists", style="yellow")
        except Links.DoesNotExist:
            Links.objects.create(
                title=title,
                tags=tags,
                url=slug,
                url_hashed=url_hashed,
                text=text,
                date_created=date_created,
                private=private)
            console.print(f"Shaarpy :: {title} added", style="magenta")


def import_pelican_folder(folder: str) -> NoReturn:
    """
    folder: folder path where to find md file to import
    """

    for p in Path(folder).glob('*.md'):
        import_pelican(folder + "/" + p.name)



# coding: utf-8
"""
    ShaarPy Test Sitemap
"""

from django.test import RequestFactory, TestCase
from django.contrib.sitemaps.views import sitemap
from shaarpy.sitemaps import ShaarpySitemap


class ShaarpySitemapTestCase(TestCase):
    """
    Test sitemap
    """

    def test_sitemaps(self):
        """
        test to check the sitemap
        """

        request = RequestFactory().get('sitemap.xml')
        kw = {'sitemaps': {'shaarpy': ShaarpySitemap}}
        response = sitemap(request, **kw)
        response['Headers'] = 'Last-Modified'
        # Check.
        self.assertEqual(response.status_code, 200)

        # @TODO triggering
        # - lastmod
        # - location



# Generated by Django 4.2.8 on 2023-12-29 15:22

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shaarpy', '0006_alter_links_date_created'),
    ]

    operations = [
        migrations.AlterField(
            model_name='links',
            name='tags',
            field=models.CharField(blank=True, max_length=255, null=True, validators=[django.core.validators.RegexValidator('^[0-9a-zA-Z\\u3131-\\uD79D\\u1100-\\u11FF\\u3130-\\u318F\\uA960-\\uA97F\\uAC00-\\uD7AF\\uD7B0-\\uD7FF,]*$', 'Only alphanumeric characters are allowed.')]),
        ),
    ]



# coding: utf-8
"""
   ShaarPy :: Views
"""
from datetime import date, datetime, timedelta, timezone
from typing import Any
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.models import User
from django.contrib.syndication.views import Feed
from django.http import HttpResponse
from django.shortcuts import redirect, render
from django.urls import reverse, reverse_lazy
from django.views.generic import (ListView, CreateView, UpdateView, DetailView, DeleteView)
from django.views.generic.base import TemplateView

import logging
import pypandoc

from shaarpy import settings
from shaarpy.forms import LinksForm, MeForm
from shaarpy.models import Links
from shaarpy.tools import (grab_full_article, small_hash, url_cleaning, _get_host)
from simple_search import search_form_factory

logger = logging.getLogger("shaarpy.views")


class SuccessMixin:
    """
    back to detail
    """
    def get_success_url(self) -> str:
        """
        redirect to the Detail object page
        """
        return reverse('link_detail', kwargs={'slug': self.object.url_hashed})


class SettingsMixin:
    """
        mixin to add settings related app data to the templates
    """
    def get_context_data(self, *, object_list=None, **kwargs):
        """
        return settings
        """
        context = super(SettingsMixin, self).get_context_data(**kwargs)
        context['SHAARPY_NAME'] = settings.SHAARPY_NAME
        context['SHAARPY_DESCRIPTION'] = settings.SHAARPY_DESCRIPTION
        context['SHAARPY_AUTHOR'] = settings.SHAARPY_AUTHOR
        context['SHAARPY_ROBOT'] = settings.SHAARPY_ROBOT

        return context


class LinksList(SettingsMixin, ListView):
    """
    home page
    """

    queryset = Links.objects.none()
    paginate_by = 10

    def get_queryset(self):
        if self.request.user.is_authenticated:
            queryset = Links.objects.all()
        else:
            queryset = Links.objects.filter(private=False)
        return queryset

    def get_context_data(self, *, object_list=None, **kwargs):
        queryset = object_list if object_list is not None else self.object_list
        page_size = self.get_paginate_by(queryset)
        context_object_name = self.get_context_object_name(queryset)

        SearchForm = search_form_factory(queryset, ['^title', 'text', 'tags'])

        search_form = SearchForm(self.request.GET or {})
        if self.request.GET.get("q"):
            if search_form.is_valid():
                queryset = search_form.get_queryset()

        context = super(LinksList, self).get_context_data(**kwargs)
        paginator, page, queryset, is_paginated = self.paginate_queryset(queryset, page_size)
        context['paginator'] = paginator
        context['page_obj'] = page
        context['is_paginated'] = is_paginated
        context['object_list'] = queryset
        context['form_search'] = SearchForm
        context['q'] = self.request.GET.get('q')
        # this will be used for the URL in the bookmarklet
        context['hostname'] = _get_host(self.request.build_absolute_uri())

        if context_object_name is not None:
            context[context_object_name] = queryset
        context.update(kwargs)

        return context


class LinksCreate(SuccessMixin, SettingsMixin, CreateView):
    """
    add a link / note
    """
    model = Links
    form_class = LinksForm

    # to deal with the popup form trigger from a javascript bookmarklet
    def get_context_data(self, *, object_list=None, **kwargs):
        context = {}
        if self.object:
            context['object'] = self.object
            context_object_name = self.get_context_object_name(self.object)
            if context_object_name:
                context[context_object_name] = self.object
        context['add_link'] = True
        context.update(kwargs)
        return super().get_context_data(**context)

    # to deal with the popup form trigger from a javascript bookmarklet

    def get_initial(self) -> dict[str, Any]:
        initial = {}
        if self.request.GET.get('post'):
            url = self.request.GET.get('post')
            url = url_cleaning(url)
            initial = {'url': url,
                       'title': self.request.GET.get('title')}
        return initial

    def form_valid(self, form):
        url = form.cleaned_data['url']
        title = form.cleaned_data['title']
        text = form.cleaned_data['text']
        url = url_cleaning(url)
        if url:
            try:
                # check if url already exist and then redirect to it
                links = Links.objects.get(url=url)
                msg = f"ShaarPy :: link already exists {url}"
                logger.debug(msg)
                return redirect('link_detail', **{'slug': links.url_hashed})
            except Links.DoesNotExist:
                pass

            # when you just want to save the URL and keep the title and body you entered
            # do not go to grab the article content at all
            if title is None and text == '':
                self.object = form.save()
                self.object.title, self.object.text, self.object.image, self.object.video = grab_full_article(url)

        else:
            self.object = form.save()
            if self.object.title is None:
                self.object.title = "Note:"

        self.object = form.save()

        # generate the tiny url
        self.object.url_hashed = small_hash(self.object.date_created.strftime("%Y%m%d_%H%M%S"))

        self.object = form.save()

        # to deal with the popup form trigger from a javascript bookmarklet
        if self.request.GET.get('source') == "bookmarklet":
            return HttpResponse('<script type="text/javascript">window.close();</script>')

        return super().form_valid(form)


class LinksUpdate(SuccessMixin, SettingsMixin, UpdateView):
    """
    update a link / note
    """

    model = Links
    form_class = LinksForm
    slug_field = 'url_hashed'

    def get_context_data(self, *, object_list=None, **kwargs):
        context = {}
        if self.object:
            context['object'] = self.object
            context_object_name = self.get_context_object_name(self.object)
            if context_object_name:
                context[context_object_name] = self.object
        context['edit_link'] = True
        context.update(kwargs)
        return super().get_context_data(**context)


class LinksDelete(SettingsMixin, DeleteView):
    """
    delete a link / note
    """

    model = Links
    success_url = reverse_lazy("home")


class LinksDetail(SettingsMixin, DetailView):
    """
    view a link / note
    """

    model = Links
    slug_field = 'url_hashed'


class PublicLinks(SettingsMixin, ListView):
    """
        Public Links List
    """
    def get_queryset(self):
        queryset = Links.objects.none()
        if self.request.user.is_authenticated:
            queryset = Links.objects.filter(private=False)
        return queryset

    def get_context_data(self, *, object_list=None, **kwargs):
        queryset = object_list if object_list is not None else self.object_list
        page_size = self.get_paginate_by(queryset)
        context_object_name = self.get_context_object_name(queryset)
        context = super(PublicLinks, self).get_context_data(**kwargs)
        paginator, page, queryset, is_paginated = self.paginate_queryset(queryset, page_size or 1)
        context['paginator'] = paginator
        context['page_obj'] = page
        context['is_paginated'] = is_paginated
        context['object_list'] = queryset
        context['audience'] = 'public'

        if context_object_name is not None:
            context[context_object_name] = queryset
        context.update(kwargs)

        return context


class PrivateLinks(SettingsMixin, ListView):
    """
        Private Links List
    """
    def get_queryset(self):
        queryset = Links.objects.none()
        if self.request.user.is_authenticated:
            queryset = Links.objects.filter(private=True)
        return queryset

    def get_context_data(self, *, object_list=None, **kwargs):
        queryset = object_list if object_list is not None else self.object_list
        page_size = self.get_paginate_by(queryset)
        context_object_name = self.get_context_object_name(queryset)
        context = super(PrivateLinks, self).get_context_data(**kwargs)
        paginator, page, queryset, is_paginated = self.paginate_queryset(queryset, page_size or 1)
        context['paginator'] = paginator
        context['page_obj'] = page
        context['is_paginated'] = is_paginated
        context['object_list'] = queryset
        context['audience'] = 'private'

        if context_object_name is not None:
            context[context_object_name] = queryset
        context.update(kwargs)

        return context


class LinksByTagList(SettingsMixin, ListView):
    """
        LinksByTag List
    """
    queryset = Links.objects.none()
    paginate_by = 10

    def get_queryset(self):
        """
            get the links with that tags
        """
        tags = None if self.kwargs['tags'] == '0Tag' else self.kwargs['tags']
        # when tags is None
        # get the data with tags is null
        if tags:
            queryset = Links.objects.filter(tags__contains=tags)
        else:
            queryset = Links.objects.filter(tags__exact=None)

        # to not return private links, reduce the queryset to public links only
        if self.request.user.is_authenticated is False:
            queryset = queryset.filter(private=False)
        return queryset

    def get_context_data(self, *, object_list=None, **kwargs):
        context = super(LinksByTagList, self).get_context_data(**kwargs)
        context['tag'] = self.kwargs['tags']
        context.update(kwargs)
        return super().get_context_data(**context)


class TagsList(SettingsMixin, ListView):
    """
        Tags List
    """
    template_name = 'shaarpy/tags_list.html'
    queryset = Links.objects.all()

    def get_context_data(self, *, object_list=None, **kwargs):
        queryset = object_list if object_list is not None else self.object_list
        context_object_name = self.get_context_object_name(queryset)
        tags = []
        for data in queryset:
            if data.tags is not None:
                for tag in data.tags.split(','):
                    tags.append(tag)
            else:
                tags.append('0Tag')
        tags = sorted(tags)
        tags_dict = {}
        for my_tag in tags:
            tags_dict.update({my_tag: tags.count(my_tag)})

        context = {
            'object_list': queryset,
            'tags': tags_dict
        }

        if context_object_name is not None:
            context[context_object_name] = queryset
        context.update(kwargs)
        return super().get_context_data(**context)


class DailyLinks(SettingsMixin, ListView):
    """
        Daily Links List
    """
    template_name = 'shaarpy/daily_list.html'
    queryset = Links.objects.none()

    def get_queryset(self):
        """
        by default, get the "yesterday" links
        """
        today = date.today()
        yesterday = today - timedelta(days=1)

        if 'yesterday' in self.kwargs:
            yesterday = self.kwargs['yesterday']

        # do not return private links
        if self.request.user.is_authenticated is False:
            queryset = Links.objects.filter(date_created__date=yesterday, private=False)
        else:
            queryset = Links.objects.filter(date_created__date=yesterday)
        return queryset

    def get_context_data(self, *, object_list=None, **kwargs):
        """
            checkout which "yesterday" it is
        """
        today = date.today()
        yesterday = today - timedelta(days=1)

        if 'yesterday' in self.kwargs:
            yesterday = self.kwargs['yesterday']
            yesterday = datetime.strptime(yesterday, '%Y-%m-%d').replace(tzinfo=timezone.utc)

        queryset = object_list if object_list is not None else self.object_list
        context_object_name = self.get_context_object_name(queryset)

        # do not return private links
        if self.request.user.is_authenticated is False:
            previous_date = Links.objects.filter(date_created__lte=yesterday,
                                                 private=False).order_by("-date_created").first()
        else:
            previous_date = Links.objects.filter(date_created__lte=yesterday).order_by("-date_created").first()

        if previous_date:
            previous_date = previous_date.date_created.date()

        # do not return private links
        if self.request.user.is_authenticated is False:
            next_date = Links.objects.filter(date_created__date__gt=yesterday,
                                             private=False).order_by("date_created").first()
        else:
            next_date = Links.objects.filter(date_created__date__gt=yesterday).order_by("date_created").first()
        if next_date:
            next_date = next_date.date_created.date()

        context = {
            'object_list': queryset,
            'previous_date': previous_date,
            'next_date': next_date,
            'current_date': yesterday
        }

        if context_object_name is not None:
            context[context_object_name] = queryset
        context.update(kwargs)
        return super().get_context_data(**context)


# the user
class Me(SettingsMixin, LoginRequiredMixin, TemplateView):

    """
        access to the profile page
    """

    template_name = "me.html"

    def get_context_data(self, *, object_list=None, **kwargs):
        context = super().get_context_data(**kwargs)
        context['object'] = self.request.user
        return context


class MeUpdate(SettingsMixin, LoginRequiredMixin, UpdateView):
    """
        Update the User profile
    """
    model = User
    form_class = MeForm
    template_name = 'edit_me.html'
    success_url = reverse_lazy('me')

    def get_object(self, queryset=None):
        """
        get only the data of the current user
        :param queryset:
        :return:
        """
        return User.objects.get(id=self.request.user.id)


# the feeds
class LatestLinksFeed(SettingsMixin, Feed):
    """
        Generate an RSS Feed
        https://docs.djangoproject.com/en/5.0/ref/contrib/syndication/
    """
    title = settings.SHAARPY_NAME
    link = "/"
    description = settings.SHAARPY_DESCRIPTION

    def items(self):
        return Links.objects.filter(private=False).order_by('-date_created')[:15]

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return pypandoc.convert_text(item.text, 'html', format='gfm')

    def item_link(self, item):
        return reverse('link_detail', args=[item.url_hashed])

    def item_pubdate(self, item):
        return item.date_created


def error_403(request, exception):
    data = {'SHAARPY_AUTHOR': settings.SHAARPY_AUTHOR,
            'SHAARPY_NAME': settings.SHAARPY_NAME,
            'SHAARPY_DESCRIPTION': settings.SHAARPY_DESCRIPTION,
            'SHAARPY_ROBOT': settings.SHAARPY_ROBOT}

    return render(request, 'shaarpy/403.html', data)


def error_404(request, exception):
    data = {'SHAARPY_AUTHOR': settings.SHAARPY_AUTHOR,
            'SHAARPY_NAME': settings.SHAARPY_NAME,
            'SHAARPY_DESCRIPTION': settings.SHAARPY_DESCRIPTION,
            'SHAARPY_ROBOT': settings.SHAARPY_ROBOT}
    return render(request, 'shaarpy/404.html', data)


def error_500(request):
    data = {'SHAARPY_AUTHOR': settings.SHAARPY_AUTHOR,
            'SHAARPY_NAME': settings.SHAARPY_NAME,
            'SHAARPY_DESCRIPTION': settings.SHAARPY_DESCRIPTION,
            'SHAARPY_ROBOT': settings.SHAARPY_ROBOT}
    return render(request, 'shaarpy/500.html', data)



# Generated by Django 4.0.1 on 2022-01-26 12:30

from django.db import migrations, models

from shaarpy.tools import small_hash


def url_hashing(apps, schema_editor):
    """
        convert classical URL to tiny one
    """
    Links = apps.get_model("shaarpy", "Links")
    db_alias = schema_editor.connection.alias
    links = Links.objects.using(db_alias).all()

    for link in links:
        link.url_hashed = small_hash(link.date_created.strftime("%Y%m%d_%H%M%S"))
        link.save()


class Migration(migrations.Migration):

    dependencies = [
        ('shaarpy', '0003_links_url_hashed_alter_links_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='links',
            name='url_hashed',
            field=models.SlugField(blank=True, max_length=10, null=True),
        ),
        migrations.RunPython(url_hashing),
    ]



# coding: utf-8
"""
    ShaarPy :: Importing Pelican Markdown folder
"""

from django.core.management.base import BaseCommand
import os
from rich.console import Console
from shaarpy.tools import import_pelican_folder

console = Console()

__author__ = 'FoxMaSk'


def load_from_folder(folder):
    """
        load Pelican Markdown from a folder
    """
    if os.path.exists(folder):
        import_pelican_folder(folder)
    else:
        console.print("provided folder does not exists", style="red")


class Command(BaseCommand):
    help = 'Import Pelican Markdown folder'

    def add_arguments(self, parser):
        parser.add_argument("folder", help="provide the path to the Markdown folder", type=str)

    def handle(self, *args, **options):
        console.print(f"Shaarpy :: Importing folder {options['folder']} in progress", style="green")
        load_from_folder(options['folder'])
        console.print(f"Shaarpy :: Importing folder {options['folder']} is finished", style="green")



"""
Django settings for shaarpy project.

Generated by 'django-admin startproject' using Django 5.0.

For more information on this file, see
https://docs.djangoproject.com/en/5.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.0/ref/settings/
"""

import os
from pathlib import Path
from django.urls import reverse_lazy
import environ


# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

env = environ.Env()

env_file_path = os.path.join(BASE_DIR, 'shaarpy', '.env')
env_file = environ.Env.read_env(env_file_path)
env.read_env(env_file)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = env.str('SECRET_KEY', default='!DONTFORGETTOCHANGETHISVALUE!')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = env.bool('DEBUG', default=False)  # set to False when using in production

ALLOWED_HOSTS = env.list('ALLOWED_HOSTS', default=['127.0.0.1', 'localhost'])

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.0/howto/deployment/checklist/


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'shaarpy',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'shaarpy.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'shaarpy.wsgi.application'


# Database
# https://docs.djangoproject.com/en/5.0/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': env.str('DB_ENGINE', default='django.db.backends.sqlite3'),
        'NAME': env.str('DB_NAME', default=BASE_DIR / 'db.sqlite3'),
        'USER': env.str('DB_USER', default=''),
        'PASSWORD': env.str('DB_PASSWORD', default=''),
        'HOST': env.str('DB_HOST', default=''),
        'PORT': env.str('DB_PORT', default=''),
    },
    'TEST': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'shaarpy-test.sqlite3',
    }
}


# Password validation
# https://docs.djangoproject.com/en/5.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format':
                '%(asctime)s %(levelname)s %(module)s %(process)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        'file': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': BASE_DIR / 'shaarpy.log',
            'maxBytes': 61280,
            'backupCount': 3,
            'formatter': 'verbose',

        },
    },
    'loggers':
    {
        'django.request': {
            'handlers': ['mail_admins', 'file'],
            'level': 'ERROR',
            'propagate': True,
        },
        'shaarpy.views': {
            'handlers': ['console', 'file'],
            'level': 'INFO',
        },
        'shaarpy.command': {
            'handlers': ['console', 'file'],
            'level': 'INFO',
        },
        'shaarpy.tools': {
            'handlers': ['console', 'file'],
            'level': 'INFO',
        },
    }
}

# Internationalization
# https://docs.djangoproject.com/en/5.0/topics/i18n/
LANGUAGE_CODE = env.str('LANGUAGE_CODE', default='en-en')
TIME_ZONE = env.str('TIME_ZONE', default='UTC')
USE_I18N = env.bool('USE_I18N', default=True)
USE_TZ = env.bool('USE_TZ', default=True)

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.0/howto/static-files/

STATIC_URL = 'static/'
STATIC_ROOT = BASE_DIR / 'static'

# Default primary key field type
# https://docs.djangoproject.com/en/5.0/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

CSRF_TRUSTED_ORIGINS = env.list('CSRF_TRUSTED_ORIGINS', default=['https://*.mydomain.com'])

LOGOUT_REDIRECT_URL = reverse_lazy('home')

SHAARPY_AUTHOR = env.str('SHAARPY_AUTHOR', default="FoxMaSk")
SHAARPY_NAME = env.str('SHAARPY_NAME', default=f"ShaarPy - {SHAARPY_AUTHOR} Links")
SHAARPY_DESCRIPTION = env.str('SHAARPY_DESCRIPTION', default="Share thoughts, links ideas, notes")
SHAARPY_ROBOT = env.str('SHAARPY_ROBOT', default="index, follow")
SHAARPY_LOCALSTORAGE_MD = env.str('SHAARPY_LOCALSTORAGE_MD', default="")
SHAARPY_STYLE = env.str('SHAARPY_STYLE', default="blue")



# coding: utf-8
"""
    ShaarPy :: Extra balls (because that reminds me pinball:)
"""
from django.urls import reverse
from django import template
import pypandoc

register = template.Library()


@register.filter(name='tags')
# draw the tags in the "card footer"
def tags(value):
    out = ''
    for tag in value.split(','):
        out += '<a href="' + reverse('links_by_tag_list', args=[tag]) + '">' \
               '<span class="badge rounded-pill text-bg-secondary">' + tag + '</span></a> '

    return out


@register.filter(name='markdown')
def markdown(text):
    # convert into Github_Flavor_Markdown
    return pypandoc.convert_text(text, "html", format="gfm")



# Generated by Django 4.0.1 on 2022-01-28 20:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shaarpy', '0004_alter_links_url_hashed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='links',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]



# coding: utf-8
"""
    ShaarPy :: Test Me
"""

from django.test import RequestFactory, TestCase
from django.contrib.auth.models import User
from django.urls import reverse

from shaarpy.views import Me, MeUpdate


class MeTestCase(TestCase):
    """
    test Me view
    """

    def setUp(self):
        super(MeTestCase, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username='foxmask', email='my@email.org', password='top_secret')

    def test_me(self):
        """
        test /me
        """
        template = "me.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('me'))
        request.user = self.user
        view = Me.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)


class MeUpdateTestCase(TestCase):
    """
    test Me Update
    """

    def setUp(self):
        super(MeUpdateTestCase, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.create_user(username='foxmask', email='my@email.org', password='top_secret')

    def test_me(self):
        """
        me view
        """
        template = "edit_me.html"
        # Setup request and view.
        request = RequestFactory().get(reverse('edit_me'))
        request.user = self.user
        view = MeUpdate.as_view(template_name=template)
        # Run.
        response = view(request)
        # Check.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name[0], template)



# coding: utf-8
"""
    Sitemap - https://docs.djangoproject.com/en/5.0/ref/contrib/sitemaps/
"""

from django.contrib.sitemaps import Sitemap
from django.urls import reverse

from shaarpy.models import Links


class ShaarpySitemap(Sitemap):
    """
    Sitemap Shaarpy sauce
    """
    changefreq = "weekly"
    priority = 0.5

    def items(self):
        return Links.objects.filter(private=False).order_by('-date_created')[:15]

    def lastmod(self, obj):
        """
        obj: object of the sitemap
        """
        return obj.date_created

    def location(self, item):
        return reverse('link_detail', kwargs={'slug': item.url_hashed})



# coding: utf-8
"""
    ShaarPy :: Admin
"""

from django.contrib import admin
from shaarpy.models import Links


class LinksAdmin(admin.ModelAdmin):
    list_display = ('title', 'url', 'tags', 'date_created', 'private')
    search_fields = ['title', 'text', 'tags']


admin.site.register(Links, LinksAdmin)



# coding: utf-8
"""
    ShaarPy
"""
from django.test import TestCase
import shaarpy


class VersionTestCase(TestCase):

    """
      check VERSION.txt
    """

    def test_version(self):

        self.assertIs(type(shaarpy.__version__), str)



# Generated by Django 4.0.1 on 2022-01-10 19:59

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Links',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tags', models.CharField(blank=True, max_length=255, null=True)),
                ('url', models.URLField(blank=True, max_length=2048, null=True)),
                ('title', models.CharField(max_length=255, blank=True, null=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('private', models.BooleanField(default=False)),
                ('image', models.TextField(null=True, blank=True)),
                ('video', models.TextField(null=True, blank=True)),
                ('date_created', models.DateTimeField(default=datetime.datetime.now)),
            ],
            options={
                'verbose_name_plural': 'Links',
                'ordering': ['-date_created'],
            },
        ),
    ]



# coding: utf-8
"""
    ShaarPy :: Exporting in markdown
"""
from django.core.management.base import BaseCommand
import logging
import os.path
from rich.console import Console
from shaarpy.models import Links
from shaarpy.tools import create_md_file
from slugify import slugify

console = Console()
logger = logging.getLogger("command")

__author__ = 'FoxMaSk'


class Command(BaseCommand):
    help = 'Export Shaarpy in Markdown files'

    def add_arguments(self, parser):
        parser.add_argument("folder", help="provide the path of the folder you want to export files", type=str)

    def handle(self, *args, **options):
        links = Links.objects.all()

        if os.path.exists(options['folder']):
            for link in links:
                file_name = slugify(link.title) + '.md'
                log = f"Shaarpy :: Exporting md file {options['folder']}/{file_name}"
                console.print(log, style="green")
                logger.info(log)
                create_md_file(options['folder'],
                               link.title,
                               link.url,
                               link.text,
                               link.tags,
                               link.date_created,
                               link.private,
                               link.image,
                               link.video)
